@@ -10,6 +10,8 @@
 
 """
 
+import logging
+
 from datalad.distribution.dataset import Dataset
 from datalad.support.exceptions import (
     IncompleteResultsError,
@@ -26,6 +28,7 @@ from datalad.tests.utils import (
     assert_status,
     DEFAULT_BRANCH,
     eq_,
+    known_failure_githubci_osx,
     neq_,
     ok_,
     ok_file_has_content,
@@ -33,6 +36,7 @@ from datalad.tests.utils import (
     skip_if_on_windows,
     skip_ssh,
     slow,
+    swallow_logs,
     with_tempfile,
     with_tree,
     SkipTest,
@@ -129,7 +133,7 @@ def check_push(annex, src_path, dst_path):
     assert_in_results(
         res, status='impossible',
         message='No push target given, and none could be auto-detected, '
-        'please specific via --to')
+        'please specify via --to')
     eq_(orig_branches, src_repo.get_branches())
     # target sibling
     target = mk_push_target(src, 'target', dst_path, annex=annex)
@@ -791,3 +795,42 @@ def test_push_git_annex_branch_when_no_data(path):
     assert_in("git-annex",
               {d["refname:strip=2"]
                for d in target.for_each_ref_(fields="refname:strip=2")})
+
+
+@known_failure_githubci_osx
+@with_tree(tree={"ds": {"f0": "0", "f1": "0", "f2": "0",
+                        "f3": "1",
+                        "f4": "2", "f5": "2"}})
+def test_push_git_annex_branch_many_paths_same_data(path):
+    path = Path(path)
+    ds = Dataset(path / "ds").create(force=True)
+    ds.save()
+    mk_push_target(ds, "target", str(path / "target"),
+                   annex=True, bare=False)
+    nbytes = sum(ds.repo.get_content_annexinfo(paths=[f])[f]["bytesize"]
+                 for f in [ds.repo.pathobj / "f0",
+                           ds.repo.pathobj / "f3",
+                           ds.repo.pathobj / "f4"])
+    with swallow_logs(new_level=logging.DEBUG) as cml:
+        res = ds.push(to="target")
+    assert_in("{} bytes of annex data".format(nbytes), cml.out)
+    # 3 files point to content already covered by another file.
+    assert_result_count(res, 3,
+                        action="copy", type="file", status="notneeded")
+
+
+@known_failure_githubci_osx
+@with_tree(tree={"ds": {"f0": "0"}})
+def test_push_matching(path):
+    path = Path(path)
+    ds = Dataset(path / "ds").create(force=True)
+    ds.config.set('push.default', 'matching', where='local')
+    ds.save()
+    remote_ds = mk_push_target(ds, 'local', str(path / 'dssibling'),
+                               annex=True, bare=False)
+    # that fact that the next one even runs makes sure that we are in a better
+    # place than https://github.com/datalad/datalad/issues/4888
+    ds.push(to='local')
+    # and we pushed the commit in the current branch
+    eq_(remote_ds.get_hexsha(DEFAULT_BRANCH),
+        ds.repo.get_hexsha(DEFAULT_BRANCH))
