@@ -27,6 +27,7 @@ from datalad.tests.utils import (
     assert_status,
     chpwd,
     create_tree,
+    DEFAULT_BRANCH,
     eq_,
     known_failure_appveyor,
     known_failure_windows,
@@ -44,7 +45,6 @@ import datalad.utils as ut
 from datalad.distribution.dataset import Dataset
 from datalad.support.annexrepo import AnnexRepo
 from datalad.support.exceptions import CommandError
-from datalad.support.external_versions import external_versions
 from datalad.api import (
     create,
     install,
@@ -154,9 +154,10 @@ def test_save_message_file(path):
                        "msg": "add foo"})
     ds.repo.add("foo")
     ds.save(message_file=op.join(ds.path, "msg"))
-    # ATTN: Use master explicitly so that this check works when we're on an
-    # adjusted branch too (e.g., when this test is executed under Windows).
-    eq_(ds.repo.format_commit("%s", "master"),
+    # ATTN: Consider corresponding branch so that this check works when we're
+    # on an adjusted branch too (e.g., when this test is executed under
+    # Windows).
+    eq_(ds.repo.format_commit("%s", DEFAULT_BRANCH),
         "add foo")
 
 
@@ -711,7 +712,7 @@ def test_surprise_subds(path):
     # this test irrelevant because it is about the unborn branch edge case.
     adjusted = somerepo.is_managed_branch()
     # This edge case goes away with Git v2.22.0.
-    fixed_git = external_versions['cmd:git'] >= '2.22.0'
+    fixed_git = somerepo.git_version >= '2.22.0'
 
     # save non-recursive
     res = ds.save(recursive=False, on_failure='ignore')
