@@ -19,11 +19,7 @@ import os.path as op
 from collections import (
     OrderedDict,
 )
-
-try:
-    from collections.abc import Mapping
-except ImportError:  # Python <= 3.3
-    from collections import Mapping
+from collections.abc import Mapping
 
 from datalad import cfg
 from datalad.interface.annotate_paths import AnnotatePaths
