@@ -16,30 +16,33 @@ from os.path import join as opj
 from datalad.api import create
 from datalad.coreapi import Dataset
 from datalad.dochelpers import exc_str
-from datalad.api import wtf
-from datalad.api import no_annex
+from datalad.api import (
+    no_annex,
+    wtf,
+)
 from datalad.plugin.wtf import _HIDDEN
 from datalad.version import __version__
 
 from ..wtf import SECTION_CALLABLES
 
 from datalad.utils import assure_unicode
-from datalad.tests.utils import swallow_outputs
-from datalad.tests.utils import with_tempfile
-from datalad.tests.utils import with_tree
-from datalad.tests.utils import chpwd
-from datalad.tests.utils import create_tree
-from datalad.tests.utils import assert_status
-from datalad.tests.utils import assert_in
-from datalad.tests.utils import assert_not_in
-from datalad.tests.utils import ok_startswith
-from datalad.tests.utils import eq_
-from datalad.tests.utils import ok_clean_git
-from datalad.tests.utils import skip_if_no_module
-from datalad.tests.utils import SkipTest
-from datalad.tests.utils import OBSCURE_FILENAME
-from datalad.tests.utils import known_failure_githubci_win
-
+from datalad.tests.utils import (
+    assert_in,
+    assert_not_in,
+    assert_repo_status,
+    assert_status,
+    chpwd,
+    create_tree,
+    eq_,
+    known_failure_githubci_win,
+    OBSCURE_FILENAME,
+    ok_startswith,
+    skip_if_no_module,
+    SkipTest,
+    swallow_outputs,
+    with_tempfile,
+    with_tree,
+)
 
 broken_plugin = """garbage"""
 
@@ -69,7 +72,7 @@ def test_wtf(topdir):
     path = opj(topdir, OBSCURE_FILENAME)
     # smoke test for now
     with swallow_outputs() as cmo:
-        wtf(dataset=path)
+        wtf(dataset=path, on_failure="ignore")
         assert_not_in('## dataset', cmo.out)
         assert_in('## configuration', cmo.out)
         # Those sections get sensored out by default now
@@ -165,7 +168,7 @@ def test_wtf(topdir):
 @with_tempfile(mkdir=True)
 def test_no_annex(path):
     ds = create(path)
-    ok_clean_git(ds.path)
+    assert_repo_status(ds.path)
     create_tree(
         ds.path,
         {'code': {
@@ -177,7 +180,7 @@ def test_no_annex(path):
     no_annex(pattern=['code/**', 'README'], dataset=ds.path)
     # add inannex and README post configuration
     ds.save([opj('code', 'notinannex'), 'README'])
-    ok_clean_git(ds.path)
+    assert_repo_status(ds.path)
     # one is annex'ed, the other is not, despite no change in add call
     # importantly, also .gitattribute is not annexed
     eq_([opj('code', 'inannex')],
@@ -209,7 +212,7 @@ def test_add_readme(path):
     ds = Dataset(path).create(force=True)
     ds.save()
     ds.aggregate_metadata()
-    ok_clean_git(ds.path)
+    assert_repo_status(ds.path)
     assert_status('ok', ds.add_readme())
     # should use default name
     eq_(
